from django.contrib.admin.templatetags.admin_list import items_for_result, result_headers
from django.template import Library
from django.forms.models import modelform_factory


register = Library()

def results(cl,r):
    qe = hasattr(cl.model_admin, 'quick_editable')
    if cl.formset:
        if qe:
            for res, form in zip(cl.result_list, cl.formset.forms):
                yield {
                    'fields':list(items_for_result(cl, res, form)), 
                    'quickedit':form
                }
        else:
            for res, form in zip(cl.result_list, cl.formset.forms):
                yield list(items_for_result(cl, res, form))
    else:
        if qe:
            for res in cl.result_list:
                yield {
                    'fields':list(items_for_result(cl, res, None)), 
                    'quickedit':cl.model_admin.get_changelist_formset(r).form(instance=res)
                }
        else:
            for res in cl.result_list:
                yield list(items_for_result(cl, res, None))

def qe_result_list(context, cl):
    return {
        'cl': cl,
        'result_headers': list(result_headers(cl)),
        'results': list(results(cl,context['request'])),
    }
qe_result_list = register.inclusion_tag('quickedit/change_list_results.html',
                        takes_context=True)(qe_result_list)



from setuptools import setup, find_packages
import quickedit


setup(name='django-quickedit',
      version=quickedit.__version__,
      description='Quick editing of fields in the django admin',
      author='Justin Quick',
      author_email='justquick@gmail.com',
      url='http://github.com/washingtontimes/django-quickedit',
      packages=find_packages(),
      #package_data={'quickedit':['quickedit/templates/quickedit/*.html']},
      include_package_data=True,
      classifiers=['Development Status :: 4 - Beta',
          'Environment :: Web Environment',
          'Framework :: Django',
          'License :: OSI Approved :: Apache Software License',
          ],
      )



from django.conf import settings
from django.db.models import get_model
from django.contrib import admin
from django.utils.functional import curry
from django.forms.models import modelformset_factory, modelform_factory


FIELDS = getattr(settings, 'QUICK_EDITABLE_FIELDS', {}).copy()
for k,v in FIELDS.items():
    if isinstance(k, basestring):
        FIELDS[get_model(*k.split('.'))] = v
        del FIELDS[k]


class QuickEditAdmin(object):
    change_list_template = 'quickedit/change_list.html'
    
    def get_changelist_formset(self, request, **kwargs):
        """
        Returns the quickedit formset for the row
        """
        defaults = {
            "formfield_callback": curry(self.formfield_for_dbfield, request=request),
        }
        defaults.update(kwargs)
        return modelformset_factory(self.model, modelform_factory(self.model),
                extra=0, fields=self.quick_editable, **defaults)

for model,modeladmin in admin.site._registry.items():
    if model in FIELDS:
        attrs = {'quick_editable': FIELDS[model]}
        if not modeladmin.list_editable:
            attrs.update(list_editable=(FIELDS[model][0],))
        admin.site.unregister(model)
        admin.site.register(model, type('newadmin', (QuickEditAdmin, modeladmin.__class__), attrs))